from .address import (
    SnapshotMCUSource,
    SnapshotRegisterAddress,
    SnapshotStartResult,
    SnapshotExportDevice,
    CommandType,
)

__all__ = [
    "SnapshotMCUSource",
    "SnapshotRegisterAddress",
    "SnapshotStartResult",
    "SnapshotExportDevice",
    "CommandType",
]



from enum import IntEnum


class SnapshotMCUSource(IntEnum):
    MCU_SOURCE_MDSP = 0
    MCU_SOURCE_SDSP = 1


class SnapshotExportDevice(IntEnum):
    EXPORT_DEVICE_UART = 0
    EXPORT_DEVICE_USB = 1


class SnapshotStartResult(IntEnum):
    SNAPSHOT_START_START = 0xAAAA
    SNAPSHOT_START_SUCCESS = 0xA5A5
    SNAPSHOT_START_FAILED = 0x5A5A
    SNAPSHOT_START_STOP = 0xBBBB


class SnapshotRegisterAddress(IntEnum):
    SNAPSHOT_REGISTERADDRESS_MCUSOURCE = 0x4011
    SNAPSHOT_REGISTERADDRESS_EXPORTDEVICE = 0x4012
    SNAPSHOT_REGISTERADDRESS_START = 0x4013
    SNAPSHOT_REGISTERADDRESS_TOTALNUMBER = 0x4070
    SNAPSHOT_REGISTERADDRESS_SNAPSHOTINDEX = 0x4071
    SNAPSHOT_REGISTERADDRESS_PACKDATASTATUS = 0x4072
    SNAPSHOT_REGISTERADDRESS_DATASIZE = 0x4073
    SNAPSHOT_REGISTERADDRESS_CHANNELNUMBER = 0x4074
    SNAPSHOT_REGISTERADDRESS_CHANNELDATASIZE = 0x4075
    SNAPSHOT_REGISTERADDRESS_PACKINDEX = 0x407F
    SNAPSHOT_REGISTERADDRESS_PACKDATA = 0x4080


class CommandType(IntEnum):
    ACK = 0x80  # 接收应答
    TOTAL_PACKETS = 0x81  # 上报总包数
    DATA_PACKET = 0x82  # 具体分包数据
    ERROR = 0x83  # 报错应答



from solax_py_library.exception import SolaxBaseError


class MissParam(SolaxBaseError):
    code = 0x2001
    message = "smart_scene__miss_param"


class OnlyPositive(MissParam):
    message = "smart_scene__only_positive"


class TimeRange(MissParam):
    message = "smart_scene__time_range"


class IrradianceOnlyPositive(MissParam):
    message = "smart_scene__irradiance_only_positive"


class ExportLimitNum(MissParam):
    message = "smart_scene__export_limit_num"


class ExportLimitPercent(MissParam):
    message = "smart_scene__export_limit_percent"


class ImportLimitNum(MissParam):
    message = "smart_scene__import_limit_num"


class ImportOnlyPositive(MissParam):
    message = "smart_scene__import_only_positive"


class SocLimit(MissParam):
    message = "smart_scene__soc_limit"


class ActivePowerLimitNum(MissParam):
    message = "smart_scene__active_power_limit_num"


class ReactivePowerLimitNum(MissParam):
    message = "smart_scene__reactive_power_limit_num"


class EnergyLimit(MissParam):
    message = "smart_scene__energy_limit"


class PowerLimitNum(MissParam):
    message = "smart_scene__power_limit_num"


class BatteryPowerLimitNum(MissParam):
    message = "smart_scene__battery_power_limit_num"


class PvOnlyGe0(MissParam):
    message = "smart_scene__pv_only_ge_0"


class CountLimit(MissParam):
    message = "smart_scene__count_limit"


class NameLengthLimit(MissParam):
    message = "smart_scene__name_length_limit"


class UniqueLimit(MissParam):
    message = "smart_scene__unique_limit"


class ElectricityPriceFailure(MissParam):
    message = "cloud__electricity_price_failure"


class WeatherFailure(MissParam):
    message = "cloud__weather_failure"



from enum import IntEnum


class AlarmLevel(IntEnum):
    """紧急告警、一般告警，状态提醒"""

    EMERGENCY = 1
    NORMAL = 2
    TIPS = 3

    def __str__(self):
        return {
            AlarmLevel.TIPS: "tips_alarm",
            AlarmLevel.NORMAL: "normal_alarm",
            AlarmLevel.EMERGENCY: "emergency_alarm",
        }.get(self)



from enum import Enum, IntEnum
from typing import List, Union, Any

from pydantic import BaseModel

from solax_py_library.device.constant.cabinet import TRENE_CABINET_ENUM
from solax_py_library.smart_scene.constant.message_entry import MESSAGE_ENTRY
from solax_py_library.smart_scene.exceptions.smart_scene import (
    ExportLimitNum,
    ImportLimitNum,
    ImportOnlyPositive,
    PowerLimitNum,
    SocLimit,
    ActivePowerLimitNum,
    ReactivePowerLimitNum,
    EnergyLimit,
    BatteryPowerLimitNum,
    PvOnlyGe0,
    ExportLimitPercent,
)


class SmartSceneUnit(IntEnum):
    PERCENT = 1
    NUM = 2


class ActionType(str, Enum):
    EMS1000 = "EMS1000"
    system = "system"


class EmsActionType(str, Enum):
    DoControl = "DoControl"


class SystemActionType(str, Enum):
    systemSwitch = "systemSwitch"
    exportControl = "exportControl"
    importControl = "importControl"
    workMode = "workMode"


class DoControl(BaseModel):
    DoNumber: int
    DoValue: int


class ActionChildData(BaseModel):
    data: List[Any]


class SystemActionChildData(ActionChildData):
    ...


class ActionItemData(BaseModel):
    def check_param(self, ctx):
        ...


class SystemActionItemData(ActionItemData):
    childType: SystemActionType
    childData: SystemActionChildData

    def to_text(self, lang, cabinet_type):
        if self.childType == SystemActionType.systemSwitch:
            switch = "off" if self.childData.data[0] == 0 else "on"
            return MESSAGE_ENTRY[self.childType][lang].format(
                MESSAGE_ENTRY[switch][lang]
            )
        elif self.childType == SystemActionType.exportControl:
            if self.childData.data[0] == 0:
                return MESSAGE_ENTRY["exportControlOff"][lang]
            else:
                switch = "on"
                mode = "total" if self.childData.data[1] == 1 else "per phase"
                unit = "kW" if self.childData.data[3] == 2 else "%"
                return MESSAGE_ENTRY[self.childType][lang].format(
                    MESSAGE_ENTRY[switch][lang],
                    MESSAGE_ENTRY[mode][lang],
                    self.childData.data[2],
                    unit,
                )
        elif self.childType == SystemActionType.importControl:
            if self.childData.data[0] == 0:
                return MESSAGE_ENTRY["importControlOff"][lang]
            else:
                if cabinet_type in TRENE_CABINET_ENUM:
                    msg = (
                        "importControl_standby"
                        if self.childData.data[1] == 0
                        else "importControl_discharge"
                    )
                    return MESSAGE_ENTRY[self.childType][lang].format(
                        MESSAGE_ENTRY["on"][lang],
                        MESSAGE_ENTRY[msg][lang],
                        self.childData.data[2],
                    )
                else:
                    return MESSAGE_ENTRY["importControl_AELIO"][lang].format(
                        MESSAGE_ENTRY["on"][lang], self.childData.data[1]
                    )
        elif self.childType == SystemActionType.workMode:
            return self.work_mode_to_text(lang)

    def work_mode_to_text(self, lang):
        work_mode = {
            0: "Self-use",
            1: "Feedin priority",
            2: "Back up mode",
            3: "Manual mode",
            4: "Peak Shaving",
            16: "VPP",
        }
        # 3: 手动（3 强充，4 强放，5 停止充放电）
        manual_mode = {
            3: "Forced charging",
            4: "Forced discharging",
            5: "Stop charging and discharging",
        }
        vpp_mode = {
            1: "Power Control Mode",
            2: "Electric Quantity Target Control Mode",
            3: "SOC Target Control Mode",
            4: "Push Power - Positive/Negative Mode",
            5: "Push Power - Zero Mode",
            6: "Self-Consume - Charge/Discharge Mode",
            7: "Self-Consume - Charge Only Mode",
            8: "PV&BAT Individual Setting – Duration Mode",
            9: "PV&BAT Individual Setting – Target SOC Mode",
        }
        value_data = self.childData.data
        # 手动模式
        if value_data[0] in [3]:
            if value_data[1] in [3, 4]:
                return MESSAGE_ENTRY[work_mode[value_data[0]]][lang].format(
                    MESSAGE_ENTRY[manual_mode[value_data[1]]][lang].format(
                        value_data[2], value_data[3]
                    )
                )
            else:
                return MESSAGE_ENTRY[work_mode[value_data[0]]][lang].format(
                    MESSAGE_ENTRY[manual_mode[value_data[1]]][lang]
                )
        elif value_data[0] in [16]:
            mode = vpp_mode[value_data[1]]
            if value_data[1] in [1, 2, 3, 8]:
                return MESSAGE_ENTRY[mode][lang].format(value_data[2], value_data[3])
            elif value_data[1] in [4]:
                return MESSAGE_ENTRY[mode][lang].format(value_data[2])
            elif value_data[1] in [5, 6, 7]:
                return MESSAGE_ENTRY[mode][lang]
            elif value_data[1] in [9]:
                return MESSAGE_ENTRY[mode][lang].format(
                    value_data[2], value_data[3], value_data[4]
                )
        else:
            return MESSAGE_ENTRY[work_mode[value_data[0]]][lang]
        return ""

    def check_param(self, ctx):
        if self.childType == SystemActionType.exportControl:
            export_power_top_limit = ctx.pop("export_power_top_limit")
            switch = self.childData.data[0]
            if not switch:
                return
            _, _, value, unit = self.childData.data
            if unit == SmartSceneUnit.NUM:
                if value > export_power_top_limit or value < 0:
                    return ExportLimitNum, {"up_limit": export_power_top_limit}
            else:
                if value < 0 or value > 110:
                    return ExportLimitPercent, {}
        elif self.childType == SystemActionType.importControl:
            import_power_top_limit = ctx.pop("import_power_top_limit", None)
            switch = self.childData.data[0]
            if not switch:
                return
            value = self.childData.data[-1]
            if import_power_top_limit is not None:
                if value > import_power_top_limit or value < 0:
                    return (
                        ImportLimitNum,
                        {"up_limit": import_power_top_limit},
                    )
            else:
                if value < 0:
                    return ImportOnlyPositive, {}
        elif self.childType == SystemActionType.workMode:
            work_mode = self.childData.data[0]
            total_power_top_limit = ctx.pop("total_power_top_limit")
            total_energy_top_limit = ctx.pop("total_energy_top_limit")
            soc_low_limit = ctx.pop("soc_low_limit")
            if work_mode == 3:  # 手动模式
                if self.childData.data[1] in [3, 4]:  # 强充或强放
                    _, _, power, soc = self.childData.data
                    if power <= 0 or power > total_power_top_limit:
                        return PowerLimitNum, {
                            "low_limit": 0,
                            "up_limit": total_power_top_limit,
                        }
                    if soc > 100 or soc < soc_low_limit:
                        return SocLimit, {"low_limit": soc_low_limit}
            elif work_mode == 16:  # VPP模式
                vpp_mode = self.childData.data[1]
                if vpp_mode == 1:
                    (
                        _,
                        _,
                        active_power,
                        reactive_power,
                    ) = self.childData.data
                    if (
                        active_power < -total_power_top_limit
                        or active_power > total_power_top_limit
                    ):
                        return (
                            ActivePowerLimitNum,
                            {
                                "low_limit": -total_power_top_limit,
                                "up_limit": total_power_top_limit,
                            },
                        )
                    if (
                        reactive_power < -total_power_top_limit
                        or reactive_power > total_power_top_limit
                    ):
                        return (
                            ReactivePowerLimitNum,
                            {
                                "low_limit": -total_power_top_limit,
                                "up_limit": total_power_top_limit,
                            },
                        )
                elif vpp_mode == 2:
                    _, _, energy, power = self.childData.data
                    if energy < 0 or energy > total_energy_top_limit:
                        return (
                            EnergyLimit,
                            {"up_limit": total_energy_top_limit},
                        )
                    if power < -total_power_top_limit or power > total_power_top_limit:
                        return (
                            PowerLimitNum,
                            {
                                "low_limit": -total_power_top_limit,
                                "up_limit": total_power_top_limit,
                            },
                        )
                elif vpp_mode == 3:
                    _, _, soc, power = self.childData.data
                    if soc < soc_low_limit or soc > 100:
                        return (
                            SocLimit,
                            {"low_limit": soc_low_limit},
                        )
                    if power < -total_power_top_limit or power > total_power_top_limit:
                        return (
                            PowerLimitNum,
                            {
                                "low_limit": -total_power_top_limit,
                                "up_limit": total_power_top_limit,
                            },
                        )
                elif vpp_mode == 4:
                    _, _, power = self.childData.data
                    if power < -total_power_top_limit or power > total_power_top_limit:
                        return (
                            BatteryPowerLimitNum,
                            {
                                "low_limit": -total_power_top_limit,
                                "up_limit": total_power_top_limit,
                            },
                        )
                elif vpp_mode == 8:
                    _, _, pv_power, bms_power = self.childData.data
                    if pv_power < 0:
                        return PvOnlyGe0, {}
                    elif (
                        bms_power < -total_power_top_limit
                        or bms_power > total_power_top_limit
                    ):
                        return (
                            BatteryPowerLimitNum,
                            {
                                "low_limit": -total_power_top_limit,
                                "up_limit": total_power_top_limit,
                            },
                        )
                elif vpp_mode == 9:
                    _, _, pv_power, bms_power, soc = self.childData.data
                    if pv_power < 0:
                        return PvOnlyGe0, {}
                    if (
                        bms_power < -total_power_top_limit
                        or bms_power > total_power_top_limit
                    ):
                        return (
                            BatteryPowerLimitNum,
                            {
                                "low_limit": -total_power_top_limit,
                                "up_limit": total_power_top_limit,
                            },
                        )
                    if soc < soc_low_limit or soc > 100:
                        return SocLimit, {"low_limit": soc_low_limit}


class EmsActionChildData(ActionChildData):
    data: List[DoControl]


class EmsActionItemData(ActionItemData):
    childType: EmsActionType
    childData: EmsActionChildData

    def to_text(self, lang, cabinet_type):
        if self.childType == EmsActionType.DoControl:
            message = ""
            for do_info in self.childData.data:
                message += MESSAGE_ENTRY[self.childType][lang].format(
                    do_info.DoNumber,
                    do_info.DoValue,
                )
            return message


class SmartSceneAction(BaseModel):
    type: ActionType
    data: List[Union[EmsActionItemData, SystemActionItemData]]

    def to_text(self, lang, cabinet_type):
        return [item.to_text(lang, cabinet_type) for item in self.data]



from abc import ABC, abstractmethod
from typing import List


class ModbusClientBase(ABC):
    @abstractmethod
    async def read_registers(self, address: int, quantity_of_x: int) -> List[int]:
        """抽象读取寄存器方法"""
        pass

    @abstractmethod
    async def write_registers(self, address: int, values: List[int]) -> bool:
        """抽象写入寄存器方法"""
        pass



import operator
from enum import IntEnum, Enum
from typing import Optional, List, Union, Any

from pydantic import BaseModel, validator

from solax_py_library.device.types.alarm import AlarmLevel
from solax_py_library.device.types.device import DeviceType
from solax_py_library.smart_scene.constant.message_entry import MESSAGE_ENTRY
from solax_py_library.smart_scene.exceptions.smart_scene import SocLimit


class LogicFunc(IntEnum):
    OR = 0
    AND = 1


class ConditionFunc(IntEnum):
    GT = 100
    LT = 101
    EQ = 102

    def function(self):
        return {
            ConditionFunc.GT: operator.gt,
            ConditionFunc.LT: operator.lt,
            ConditionFunc.EQ: operator.eq,
        }.get(self)


class RepeatFunc(IntEnum):
    ONCE = 103
    EVERYDAY = 104
    WEEKDAY = 105
    WEEKEND = 106
    CUSTOM = 107


class ConditionType(str, Enum):
    date = "date"
    weather = "weather"
    buyingPrice = "buyingPrice"
    sellingPrice = "sellingPrice"
    systemCondition = "systemCondition"
    cabinet = "cabinet"


class WeatherConditionType(str, Enum):
    irradiance = "irradiance"
    temperature = "temperature"


class PriceConditionType(str, Enum):
    price = "price"
    lowerPrice = "lowerPrice"
    higherPrice = "higherPrice"
    expensiveHours = "expensiveHours"
    cheapestHours = "cheapestHours"


class DateConditionType(str, Enum):
    time = "time"
    duration = "duration"


class SystemConditionType(str, Enum):
    systemSoc = "systemSoc"
    systemImportPower = "systemImportPower"  # 买电功率
    systemExportPower = "systemExportPower"  # 馈电功率


class CabinetConditionType(str, Enum):
    cabinetAlarm = "cabinetAlarm"
    cabinetSoc = "cabinetSoc"


class SmartSceneUnit(IntEnum):
    PERCENT = 1
    NUM = 2


class ConditionItemChildData(BaseModel):
    function: Optional[ConditionFunc]
    data: List[Any]


class ConditionItemData(BaseModel):
    def check_param(self, ctx):
        ...


class PriceConditionItemData(ConditionItemData):
    childType: PriceConditionType
    childData: ConditionItemChildData

    @validator("childData", always=True)
    def _check_child_data(cls, value, values):
        child_type = values.get("childType")
        if child_type in {
            PriceConditionType.lowerPrice,
            PriceConditionType.higherPrice,
        }:
            assert value.data[0] > 0, ValueError
        elif child_type in {
            PriceConditionType.expensiveHours,
            PriceConditionType.cheapestHours,
        }:
            assert 1 <= value.data[2] <= 24, ValueError
        return value

    def to_text(self, lang, unit):
        data = self.childData.data
        func = self.childData.function
        if self.childType == PriceConditionType.price:
            return MESSAGE_ENTRY[self.childType][lang].format(
                MESSAGE_ENTRY[str(func.value)][lang], f"{data[1]:.5f}", unit
            )
        elif self.childType in {
            PriceConditionType.lowerPrice,
            PriceConditionType.higherPrice,
        }:
            return MESSAGE_ENTRY[self.childType][lang].format(
                f"{data[0]:.5f}", "%" if data[1] == 1 else unit
            )
        elif self.childType in {
            PriceConditionType.expensiveHours,
            PriceConditionType.cheapestHours,
        }:
            return MESSAGE_ENTRY[self.childType][lang].format(data[0], data[1], data[2])


class SystemConditionItemData(ConditionItemData):
    childType: SystemConditionType
    childData: ConditionItemChildData

    @validator("childData", always=True)
    def _check_child_data(cls, value, values):
        child_type = values.get("childType")
        if child_type in {
            SystemConditionType.systemExportPower,
            SystemConditionType.systemImportPower,
        }:
            assert 0 <= value.data[0] <= 100000, ValueError
            value.data[0] = round(value.data[0], 2)  # 功率保留两位小数
        return value

    def to_text(self, lang, unit):
        data = self.childData.data
        func = self.childData.function
        if self.childType == SystemConditionType.systemSoc:
            return MESSAGE_ENTRY[self.childType][lang].format(
                MESSAGE_ENTRY[str(func.value)][lang], data[0]
            )
        elif self.childType in {
            SystemConditionType.systemImportPower,
            SystemConditionType.systemExportPower,
        }:
            return MESSAGE_ENTRY[self.childType][lang].format(
                MESSAGE_ENTRY[str(func.value)][lang], data[0]
            )

    def check_param(self, ctx):
        soc_low_limit = ctx.get("soc_low_limit", 5)
        if self.childType == SystemConditionType.systemSoc:
            soc = self.childData.data[0]
            if soc < soc_low_limit or soc > 100:
                return SocLimit, {"low_limit": soc_low_limit}


class CabinetConditionItemData(ConditionItemData):
    childType: CabinetConditionType
    childData: ConditionItemChildData

    @validator("childData", always=True)
    def _check_child_data(cls, value, values):
        child_type = values.get("childType")
        if child_type == CabinetConditionType.cabinetAlarm:
            assert value.data[-1] in {
                AlarmLevel.TIPS,
                AlarmLevel.NORMAL,
                AlarmLevel.EMERGENCY,
            }, ValueError
        return value

    def to_text(self, lang, unit):
        data = self.childData.data
        func = self.childData.function
        if self.childType == CabinetConditionType.cabinetSoc:
            return MESSAGE_ENTRY[self.childType][lang].format(
                MESSAGE_ENTRY[str(func.value)][lang], data[0]
            )
        elif self.childType == CabinetConditionType.cabinetAlarm:
            return MESSAGE_ENTRY[self.childType][lang].format(
                ",".join(
                    [
                        MESSAGE_ENTRY[str(DeviceType(device_type))][lang]
                        for device_type in data[:-1]
                    ]
                ),
                MESSAGE_ENTRY[str(AlarmLevel(data[-1]))][lang],
            )

    def check_param(self, ctx):
        soc_low_limit = ctx.get("soc_low_limit", 5)
        if self.childType == CabinetConditionType.cabinetSoc:
            soc = self.childData.data[0]
            if soc < soc_low_limit or soc > 100:
                return SocLimit, {"low_limit": soc_low_limit}


class DateConditionItemData(ConditionItemData):
    childType: DateConditionType
    childData: ConditionItemChildData

    @validator("childData", always=True)
    def _check_child_data(cls, value, values):
        child_type = values.get("childType")
        data = value.data
        if child_type == DateConditionType.time:
            assert isinstance(data[0], str), ValueError
        elif child_type == DateConditionType.duration:
            assert isinstance(data[0], int), ValueError
        return value

    def to_text(self, lang, unit):
        if self.childType == DateConditionType.duration:
            return MESSAGE_ENTRY[self.childType][lang].format(self.childData.data[0])
        elif self.childType == DateConditionType.time:
            return MESSAGE_ENTRY[self.childType][lang] + "/" + self.childData.data[0]


class WeatherConditionItemData(ConditionItemData):
    childType: WeatherConditionType
    childData: ConditionItemChildData

    @validator("childData", always=True)
    def _check_child_data(cls, value, values):
        child_type = values.get("childType")
        if child_type == WeatherConditionType.irradiance:
            assert value.data[0] > 0, ValueError
            assert 0 <= value.data[1] <= 24, ValueError
        return value

    def to_text(self, lang, unit):
        func = self.childData.function
        data = self.childData.data
        if self.childType == WeatherConditionType.irradiance:
            return MESSAGE_ENTRY[self.childType][lang].format(
                MESSAGE_ENTRY[str(func.value)][lang], data[0], data[1]
            )
        else:
            return MESSAGE_ENTRY[self.childType][lang].format(
                MESSAGE_ENTRY[str(func.value)][lang], data[0]
            )


class ConditionItem(BaseModel):
    type: ConditionType
    cabinet: Optional[List[str]]
    data: List[
        Union[
            DateConditionItemData,
            WeatherConditionItemData,
            PriceConditionItemData,
            SystemConditionItemData,
            CabinetConditionItemData,
        ]
    ]

    @validator("cabinet")
    def _check_cabinet(cls, value, values):
        condition_type = values.get("type")
        if condition_type == ConditionType.cabinet:
            assert value, "cabinet is None"
        return value

    def to_text(self, lang, unit):
        if self.type != ConditionType.cabinet:
            return {self.type: [d.to_text(lang, unit) for d in self.data]}
        elif self.type == ConditionType.cabinet:
            cabinet_sns = ",".join(self.cabinet)
            return {
                self.type: [d.to_text(lang, unit) for d in self.data] + [cabinet_sns]
            }


class SmartSceneCondition(BaseModel):
    operation: LogicFunc
    value: List[ConditionItem]

    def to_text(self, lang, unit):
        ret = {"operation": [MESSAGE_ENTRY[self.operation.name][lang]]}
        for v in self.value:
            ret.update(v.to_text(lang, unit))
        return ret

    def get_duration_info(self):
        for item in self.value:
            if item.type != ConditionType.date:
                continue
            for date_item in item.data:
                if date_item.childType == DateConditionType.duration:
                    return (date_item.childData.data[0] // 10) + 1
        return 1



import json
import uuid
from typing import Optional, List

from pydantic import Field
from pydantic.main import BaseModel

from solax_py_library.device.constant.cabinet import TRENE_CABINET_ENUM
from solax_py_library.smart_scene.constant.message_entry import MESSAGE_ENTRY
from solax_py_library.smart_scene.types.action import (
    SmartSceneAction,
    ActionType,
    SystemActionType,
)
from solax_py_library.smart_scene.types.condition import (
    RepeatFunc,
    SmartSceneCondition,
    LogicFunc,
    ConditionType,
    PriceConditionType,
    ConditionFunc,
)


class SmartSceneOtherInfo(BaseModel):
    duration_times: Optional[int] = Field(
        description="如果需要判断持续时间，记录当前已持续的时间", default=0
    )
    version: Optional[int] = Field(
        description="当前版本，现用于是否做数据迁移，V009版本开始", default=9
    )
    once_flag: Optional[bool] = Field(
        description="是否是单次执行，默认False", default=False
    )

    @classmethod
    def new_other_info(cls):
        return SmartSceneOtherInfo(
            duration_times=0,
            version=9,
            once_flag=False,
        ).dict()


class SmartSceneContent(BaseModel):
    name: str = Field(description="Scene name", max_length=100)
    repeatFunction: Optional[RepeatFunc] = Field(description="重复规则")
    weekList: Optional[List[int]]
    conditions: SmartSceneCondition = Field(alias="if")
    thenActions: List[SmartSceneAction] = Field(alias="then")
    elseThenActions: Optional[List[SmartSceneAction]] = Field(alias="elseThen")

    def set_copy_name(self):
        self.name = self.name + "-copy"

    def build_smart_scene(self):
        return {
            "scene_id": str(uuid.uuid4()),
            "switch": False,
            "content": json.dumps(
                {
                    "name": self.name,
                    "repeatFunction": self.repeatFunction,
                    "weekList": self.weekList,
                    "if": self.conditions.dict(),
                    "then": [item.dict() for item in self.thenActions],
                    "elseThen": [item.dict() for item in self.elseThenActions]
                    if self.elseThenActions
                    else None,
                }
            ),
            "other": json.dumps(SmartSceneOtherInfo.new_other_info()),
        }

    @classmethod
    def rec_scene(cls, lang, cabinet_type):
        scene_rec_1 = {
            "name": MESSAGE_ENTRY["rec_name_01"][lang],
            "instruction": MESSAGE_ENTRY["instruction_01"][lang],
            "repeatFunction": RepeatFunc.EVERYDAY,
            "weekList": [1, 2, 3, 4, 5, 6, 7],
            "if": {
                "operation": LogicFunc.AND,
                "value": [
                    {
                        "type": ConditionType.buyingPrice,
                        "data": [
                            {
                                "childType": PriceConditionType.price,
                                "childData": {
                                    "function": ConditionFunc.GT,
                                    "data": [1],
                                },
                            }
                        ],
                    }
                ],
            },
            "then": [
                {
                    "type": ActionType.system,
                    "data": [
                        {
                            "childType": SystemActionType.workMode,
                            "childData": {
                                "data": [16, 9, 0, -60, 100],
                            },
                        }
                    ],
                }
            ],
            "elseThen": [],
        }
        scene_rec_2 = {
            "name": MESSAGE_ENTRY["rec_name_02"][lang],
            "instruction": MESSAGE_ENTRY["instruction_02"][lang],
            "repeatFunction": RepeatFunc.EVERYDAY,
            "weekList": [1, 2, 3, 4, 5, 6, 7],
            "if": {
                "operation": LogicFunc.AND,
                "value": [
                    {
                        "type": ConditionType.sellingPrice,
                        "data": [
                            {
                                "childType": PriceConditionType.price,
                                "childData": {
                                    "function": ConditionFunc.LT,
                                    "data": [1],
                                },
                            }
                        ],
                    }
                ],
            },
            "then": [
                {
                    "type": ActionType.system,
                    "data": [
                        {
                            "childType": SystemActionType.exportControl,
                            "childData": {
                                "data": [1, 2, 0, 1],
                            },
                        }
                    ],
                }
            ],
            "elseThen": [],
        }
        if cabinet_type in TRENE_CABINET_ENUM:
            return [scene_rec_2]
        else:
            return [scene_rec_1, scene_rec_2]

    def get_weekday_index(self):
        if self.repeatFunction == RepeatFunc.EVERYDAY:
            return [1, 2, 3, 4, 5, 6, 7]
        elif self.repeatFunction == RepeatFunc.WEEKDAY:
            return [1, 2, 3, 4, 5]
        elif self.repeatFunction == RepeatFunc.WEEKEND:
            return [6, 7]
        elif self.repeatFunction == RepeatFunc.CUSTOM:
            return self.weekList

    def get_condition_types(self):
        condition_types = set()
        for v in self.conditions.value:
            condition_types.add(v.type)
        return condition_types

    def repeat_text_info(self, lang):
        if self.repeatFunction != RepeatFunc.CUSTOM:
            return MESSAGE_ENTRY[str(self.repeatFunction.value)][lang]
        else:
            return ",".join([MESSAGE_ENTRY[str(d)][lang] for d in self.weekList])
